import { useState } from "react";
import { useForm } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";
import { z } from "zod";
import { useLocation } from "wouter";
import { useToast } from "@/hooks/use-toast";
import { insertListingSchema } from "@shared/schema";
import { useQuery, useMutation } from "@tanstack/react-query";
import { apiRequest, queryClient } from "@/lib/queryClient";
import { LOCATIONS } from "@/lib/constants";
import { 
  Form, 
  FormControl, 
  FormDescription, 
  FormField, 
  FormItem, 
  FormLabel, 
  FormMessage 
} from "@/components/ui/form";
import { Input } from "@/components/ui/input";
import { Textarea } from "@/components/ui/textarea";
import { 
  Select, 
  SelectContent, 
  SelectItem, 
  SelectTrigger, 
  SelectValue 
} from "@/components/ui/select";
import { Checkbox } from "@/components/ui/checkbox";
import { Button } from "@/components/ui/button";
import { Card, CardContent } from "@/components/ui/card";
import { Loader2 } from "lucide-react";

// Extend the listing schema with validation
const listingFormSchema = insertListingSchema.extend({
  images: z.instanceof(FileList).optional(),
}).omit({ userId: true });

type ListingFormValues = z.infer<typeof listingFormSchema>;

interface ListingFormProps {
  initialData?: any;
  isEditing?: boolean;
}

const ListingForm = ({ initialData, isEditing = false }: ListingFormProps) => {
  const [, navigate] = useLocation();
  const { toast } = useToast();
  const [selectedImages, setSelectedImages] = useState<File[]>([]);
  const [isUploading, setIsUploading] = useState(false);
  
  // Fetch categories for the dropdown
  const { data: categoriesData, isLoading: isCategoriesLoading } = useQuery<{ categories: any[] }>({
    queryKey: ['/api/categories'],
  });
  
  // Form definition with zod validation
  const form = useForm<ListingFormValues>({
    resolver: zodResolver(listingFormSchema),
    defaultValues: initialData || {
      title: "",
      description: "",
      price: undefined,
      location: "Pointe-à-Pitre",
      categoryId: undefined,
      isUrgent: false,
      status: "active",
    },
  });
  
  // Create listing mutation
  const createListingMutation = useMutation({
    mutationFn: async (values: ListingFormValues) => {
      // First create the listing
      const { images, ...listingData } = values;
      const response = await apiRequest('POST', '/api/listings', listingData);
      const data = await response.json();
      
      if (images && images.length > 0) {
        setIsUploading(true);
        
        // Upload each selected image
        for (let i = 0; i < selectedImages.length; i++) {
          const formData = new FormData();
          formData.append('image', selectedImages[i]);
          
          await apiRequest('POST', `/api/listings/${data.listing.id}/images`, formData);
        }
        
        setIsUploading(false);
      }
      
      return data;
    },
    onSuccess: (data) => {
      queryClient.invalidateQueries({ queryKey: ['/api/listings'] });
      toast({
        title: "Annonce créée",
        description: "Votre annonce a été publiée avec succès.",
      });
      navigate(`/listings/${data.listing.id}`);
    },
    onError: (error) => {
      toast({
        title: "Erreur",
        description: "Une erreur est survenue lors de la création de l'annonce.",
        variant: "destructive",
      });
    },
  });
  
  // Update listing mutation
  const updateListingMutation = useMutation({
    mutationFn: async (values: ListingFormValues) => {
      const { images, ...listingData } = values;
      const response = await apiRequest('PATCH', `/api/listings/${initialData.id}`, listingData);
      const data = await response.json();
      
      if (images && images.length > 0) {
        setIsUploading(true);
        
        // Upload each selected image
        for (let i = 0; i < selectedImages.length; i++) {
          const formData = new FormData();
          formData.append('image', selectedImages[i]);
          
          await apiRequest('POST', `/api/listings/${initialData.id}/images`, formData);
        }
        
        setIsUploading(false);
      }
      
      return data;
    },
    onSuccess: (data) => {
      queryClient.invalidateQueries({ queryKey: ['/api/listings'] });
      queryClient.invalidateQueries({ queryKey: [`/api/listings/${initialData.id}`] });
      toast({
        title: "Annonce mise à jour",
        description: "Votre annonce a été mise à jour avec succès.",
      });
      navigate(`/listings/${initialData.id}`);
    },
    onError: (error) => {
      toast({
        title: "Erreur",
        description: "Une erreur est survenue lors de la mise à jour de l'annonce.",
        variant: "destructive",
      });
    },
  });
  
  // Handle form submission
  const onSubmit = (values: ListingFormValues) => {
    if (isEditing) {
      updateListingMutation.mutate(values);
    } else {
      createListingMutation.mutate(values);
    }
  };
  
  // Handle image selection
  const handleImageChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    if (e.target.files && e.target.files.length > 0) {
      const filesArray = Array.from(e.target.files);
      setSelectedImages(filesArray);
      
      // Set the images in the form
      form.setValue('images', e.target.files, { shouldValidate: true });
    }
  };
  
  const isSubmitting = createListingMutation.isPending || updateListingMutation.isPending || isUploading;
  
  return (
    <Card>
      <CardContent className="pt-6">
        <Form {...form}>
          <form onSubmit={form.handleSubmit(onSubmit)} className="space-y-6">
            <FormField
              control={form.control}
              name="categoryId"
              render={({ field }) => (
                <FormItem>
                  <FormLabel>Catégorie</FormLabel>
                  <Select 
                    onValueChange={(value) => field.onChange(parseInt(value))} 
                    defaultValue={field.value?.toString()}
                  >
                    <FormControl>
                      <SelectTrigger>
                        <SelectValue placeholder="Sélectionnez une catégorie" />
                      </SelectTrigger>
                    </FormControl>
                    <SelectContent>
                      {isCategoriesLoading ? (
                        <SelectItem value="0" disabled>Chargement des catégories...</SelectItem>
                      ) : (
                        categoriesData?.categories.map((category) => (
                          <SelectItem key={category.id} value={category.id.toString()}>
                            {category.name}
                          </SelectItem>
                        ))
                      )}
                    </SelectContent>
                  </Select>
                  <FormDescription>
                    Choisissez la catégorie qui correspond le mieux à votre annonce.
                  </FormDescription>
                  <FormMessage />
                </FormItem>
              )}
            />
            
            <FormField
              control={form.control}
              name="title"
              render={({ field }) => (
                <FormItem>
                  <FormLabel>Titre de l'annonce</FormLabel>
                  <FormControl>
                    <Input placeholder="Ex: iPhone 12 Pro - 128Go" {...field} />
                  </FormControl>
                  <FormDescription>
                    Un titre clair et concis pour attirer l'attention des acheteurs.
                  </FormDescription>
                  <FormMessage />
                </FormItem>
              )}
            />
            
            <FormField
              control={form.control}
              name="description"
              render={({ field }) => (
                <FormItem>
                  <FormLabel>Description</FormLabel>
                  <FormControl>
                    <Textarea 
                      placeholder="Décrivez votre article (état, caractéristiques, etc.)" 
                      className="min-h-[120px]"
                      {...field} 
                    />
                  </FormControl>
                  <FormDescription>
                    Soyez précis dans votre description pour augmenter vos chances de vente.
                  </FormDescription>
                  <FormMessage />
                </FormItem>
              )}
            />
            
            <FormField
              control={form.control}
              name="price"
              render={({ field }) => (
                <FormItem>
                  <FormLabel>Prix (€)</FormLabel>
                  <FormControl>
                    <div className="relative">
                      <Input 
                        type="number" 
                        placeholder="Ex: 100" 
                        className="pl-8" 
                        {...field}
                        onChange={(e) => {
                          const value = e.target.value === '' ? undefined : Number(e.target.value);
                          field.onChange(value);
                        }}
                        value={field.value === undefined ? '' : field.value}
                      />
                      <div className="absolute inset-y-0 left-0 pl-3 flex items-center pointer-events-none">
                        <span className="text-gray-500">€</span>
                      </div>
                    </div>
                  </FormControl>
                  <FormDescription>
                    Indiquez le prix en euros. Laissez vide si le prix est à débattre.
                  </FormDescription>
                  <FormMessage />
                </FormItem>
              )}
            />
            
            <FormField
              control={form.control}
              name="location"
              render={({ field }) => (
                <FormItem>
                  <FormLabel>Localisation</FormLabel>
                  <Select onValueChange={field.onChange} defaultValue={field.value}>
                    <FormControl>
                      <SelectTrigger>
                        <SelectValue placeholder="Sélectionnez une ville" />
                      </SelectTrigger>
                    </FormControl>
                    <SelectContent>
                      {LOCATIONS.filter(loc => loc !== "Toute la Guadeloupe").map((location) => (
                        <SelectItem key={location} value={location}>
                          {location}
                        </SelectItem>
                      ))}
                    </SelectContent>
                  </Select>
                  <FormDescription>
                    Indiquez où se trouve votre article.
                  </FormDescription>
                  <FormMessage />
                </FormItem>
              )}
            />
            
            <FormField
              control={form.control}
              name="isUrgent"
              render={({ field }) => (
                <FormItem className="flex flex-row items-start space-x-3 space-y-0 rounded-md border p-4">
                  <FormControl>
                    <Checkbox
                      checked={field.value}
                      onCheckedChange={field.onChange}
                    />
                  </FormControl>
                  <div className="space-y-1 leading-none">
                    <FormLabel>Annonce urgente</FormLabel>
                    <FormDescription>
                      Marquez cette annonce comme urgente pour plus de visibilité.
                    </FormDescription>
                  </div>
                </FormItem>
              )}
            />
            
            <div>
              <FormLabel htmlFor="images">Photos</FormLabel>
              <div className="border-2 border-dashed border-gray-300 rounded-md p-4 mt-2 text-center">
                <div className="mb-2">
                  <i className="fas fa-camera text-3xl text-gray-400"></i>
                </div>
                <p className="text-sm text-gray-500 mb-1">Glissez vos photos ici ou</p>
                <Button type="button" variant="link" className="text-accent text-sm font-medium">
                  <label htmlFor="images" className="cursor-pointer">Parcourir</label>
                </Button>
                <input 
                  id="images" 
                  type="file" 
                  multiple 
                  className="hidden" 
                  accept="image/*"
                  onChange={handleImageChange}
                />
                
                {selectedImages.length > 0 && (
                  <div className="mt-4">
                    <p className="text-sm text-gray-700">{selectedImages.length} image(s) sélectionnée(s)</p>
                    <div className="grid grid-cols-3 gap-2 mt-2">
                      {selectedImages.map((file, index) => (
                        <div key={index} className="relative h-20 bg-gray-100 rounded">
                          <img 
                            src={URL.createObjectURL(file)} 
                            alt={`Preview ${index}`}
                            className="h-full w-full object-cover rounded"
                          />
                        </div>
                      ))}
                    </div>
                  </div>
                )}
              </div>
              {form.formState.errors.images && (
                <p className="text-sm text-red-500 mt-1">{form.formState.errors.images.message}</p>
              )}
            </div>
            
            <Button 
              type="submit" 
              className="w-full bg-primary hover:bg-primary/90" 
              disabled={isSubmitting}
            >
              {isSubmitting && <Loader2 className="mr-2 h-4 w-4 animate-spin" />}
              {isEditing ? "Mettre à jour mon annonce" : "Publier mon annonce"}
            </Button>
          </form>
        </Form>
      </CardContent>
    </Card>
  );
};

export default ListingForm;
